/****************************************************************************

    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef VARISOURCE_H_INCLUDED
#define VARISOURCE_H_INCLUDED

#include <stk/SineWave.h>
#include <stk/BlitSaw.h>
#include <stk/BlitSquare.h>
#include <stk/Blit.h>
#include <stk/Noise.h>
#include <stk/ADSR.h>
#include <memory>

class INewtSettings;

class VariSource : public stk::Generator
{
    public:

    enum Type
    {
        SINE,
        SAW,
        SQUARE,
        IMPULSE,
        NOISE,
        #ifdef INCLUDE_EXT_INPUTS
        EXTERNAL,
        #endif
        UNDEFINED
    };

    enum Use
    {
        FLOOR,
        GMOD
    };

    VariSource(INewtSettings *settings = NULL);
    virtual ~VariSource();
    void copy(const VariSource& source);
    stk::StkFrames& tick(stk::StkFrames& frames, unsigned int channel = 0);
    stk::StkFloat tick(unsigned int frameIdx, unsigned int channel = 0);
    void setType(Type type);
    void setFrequency(stk::StkFloat frequency, bool reset = false);
    void setHarmonics(int numHarmonics);
    void reset();
    inline stk::StkFloat getFrequency() const { return _frequency; };
    inline Type getType() const { return _type; };
    inline void setAmplitude(stk::StkFloat val) { _amplitude = val; };
    inline stk::ADSR& getEnv() { return _envelope; };
    inline void setUsingEnvelope(bool val) { _usingEnvelope = val; };
    inline bool isUsingEnvelope() { return _usingEnvelope; }
    static Type getTypeFromFloat(stk::StkFloat val);
    inline void setSettings(INewtSettings *settings) { _set = settings; }
    inline void setUse(Use use) { _use = use; }
    inline stk::StkFloat lastOut() const { return _generator->lastFrame()[0]; };

    private:

    typedef std::auto_ptr<stk::Generator> GenPtr;

    Type _type;
    GenPtr _generator;
    stk::StkFloat _frequency;
    stk::StkFloat _amplitude;
    stk::ADSR _envelope;
    Use _use;
    bool _usingEnvelope;
    INewtSettings* _set;
};

#endif // VARISOURCE_H_INCLUDED
